@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -23,15 +21,17 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*	CFLocale.h
-	Copyright (c) 2002-2003, Apple, Inc. All rights reserved.
+	Copyright (c) 2002-2005, Apple, Inc. All rights reserved.
 */
 
 #if !defined(__COREFOUNDATION_CFLOCALE__)
 #define __COREFOUNDATION_CFLOCALE__ 1
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
-
 #include <CoreFoundation/CFBase.h>
+#include <CoreFoundation/CFArray.h>
+#include <CoreFoundation/CFDictionary.h>
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
 
 #if defined(__cplusplus)
 extern "C" {
@@ -39,6 +39,8 @@ extern "C" {
 
 typedef const struct __CFLocale *CFLocaleRef;
 
+
+
 #if defined(__cplusplus)
 }
 #endif
